#ifndef WEENSYOS_SCHEDOS_KERN_H
#define WEENSYOS_SCHEDOS_KERN_H
#include "schedos.h"
#include "x86struct.h"

// The process descriptor type
typedef struct process {
	registers_t p_registers;	// Current process state: registers,
					// stack location, EIP, etc.
					// 'registers_t' defined in x86struct.h
	
	int p_runnable;			// Is this process runnable? (0 or 1)
	int p_exit_status;		// Process's exit status
} process_t;


// Clock frequency: the clock interrupt, if any, happens HZ times a second
#define HZ			100

// The trap number corresponding to the first hardware interrupt
#define TRAP_INTERRUPTS		32
#define TRAP_CLOCKINTERRUPT	(TRAP_INTERRUPTS + 0)

// Top of the kernel stack
#define KERNEL_STACK_TOP	0x200000

// Functions defined in schedos-kern.c
void trap(registers_t *reg);
void schedule(void);

// Functions defined in schedos-x86.c
void interrupt_controller_init(bool_t allow_clock_interrupt);
void segment_descriptors_init(void);
void console_clear(void);
void process_init(process_t *proc);
void run(process_t *proc) __attribute__((noreturn));

// Helper functions
void *memcpy(void *dst, const void *src, size_t n);
void *memset(void *v, int c, size_t n);

#endif
